@@ -5,6 +5,8 @@
 #ifndef XGBOOST_BUILD_CONFIG_H_
 #define XGBOOST_BUILD_CONFIG_H_
 
+// These check are for Makefile.
+#if !defined(XGBOOST_MM_PREFETCH_PRESENT) && !defined(XGBOOST_BUILTIN_PREFETCH_PRESENT)
 /* default logic for software pre-fetching */
 #if (defined(_MSC_VER) && (defined(_M_IX86) || defined(_M_AMD64))) || defined(__INTEL_COMPILER)
 // Enable _mm_prefetch for Intel compiler and MSVC+x86
@@ -15,4 +17,6 @@
 #define XGBOOST_BUILTIN_PREFETCH_PRESENT
 #endif  // GUARDS
 
+#endif  // !defined(XGBOOST_MM_PREFETCH_PRESENT) && !defined()
+
 #endif  // XGBOOST_BUILD_CONFIG_H_
