@@ -17,9 +17,6 @@
 #include <stdint.h>
 #endif  // __cplusplus
 
-// XGBoost C API will include APIs in Rabit C API
-#include <rabit/c_api.h>
-
 #if defined(_MSC_VER) || defined(_WIN32)
 #define XGB_DLL XGB_EXTERN_C __declspec(dllexport)
 #else
@@ -149,23 +146,6 @@ XGB_DLL int XGDMatrixCreateFromCSREx(const size_t* indptr,
                                      size_t nelem,
                                      size_t num_col,
                                      DMatrixHandle* out);
-/*!
- * \deprecated
- * \brief create a matrix content from CSR format
- * \param indptr pointer to row headers
- * \param indices findex
- * \param data fvalue
- * \param nindptr number of rows in the matrix + 1
- * \param nelem number of nonzero elements in the matrix
- * \param out created dmatrix
- * \return 0 when success, -1 when failure happens
- */
-XGB_DLL int XGDMatrixCreateFromCSR(const bst_ulong *indptr,
-                                   const unsigned *indices,
-                                   const float *data,
-                                   bst_ulong nindptr,
-                                   bst_ulong nelem,
-                                   DMatrixHandle *out);
 /*!
  * \brief create a matrix content from CSC format
  * \param col_ptr pointer to col headers
@@ -184,23 +164,7 @@ XGB_DLL int XGDMatrixCreateFromCSCEx(const size_t* col_ptr,
                                      size_t nelem,
                                      size_t num_row,
                                      DMatrixHandle* out);
-/*!
- * \deprecated
- * \brief create a matrix content from CSC format
- * \param col_ptr pointer to col headers
- * \param indices findex
- * \param data fvalue
- * \param nindptr number of rows in the matrix + 1
- * \param nelem number of nonzero elements in the matrix
- * \param out created dmatrix
- * \return 0 when success, -1 when failure happens
- */
-XGB_DLL int XGDMatrixCreateFromCSC(const bst_ulong *col_ptr,
-                                   const unsigned *indices,
-                                   const float *data,
-                                   bst_ulong nindptr,
-                                   bst_ulong nelem,
-                                   DMatrixHandle *out);
+
 /*!
  * \brief create matrix content from dense matrix
  * \param data pointer to the data space
