@@ -276,13 +276,21 @@ class TCPSocket : public Socket{
    * \brief enable/disable TCP keepalive
    * \param keepalive whether to set the keep alive option on
    */
-  inline void SetKeepAlive(bool keepalive) {
+  void SetKeepAlive(bool keepalive) {
     int opt = static_cast<int>(keepalive);
     if (setsockopt(sockfd, SOL_SOCKET, SO_KEEPALIVE,
                    reinterpret_cast<char*>(&opt), sizeof(opt)) < 0) {
       Socket::Error("SetKeepAlive");
     }
   }
+  inline void SetLinger(int timeout = 0) {
+    struct linger sl;
+    sl.l_onoff = 1;    /* non-zero value enables linger option in kernel */
+    sl.l_linger = timeout;    /* timeout interval in seconds */
+    if (setsockopt(sockfd, SOL_SOCKET, SO_LINGER, reinterpret_cast<char*>(&sl), sizeof(sl)) == -1) {
+      Socket::Error("SO_LINGER");
+    }
+  }
   /*!
    * \brief create the socket, call this before using socket
    * \param af domain
