@@ -61,6 +61,10 @@ bst_float SplitEvaluator::ComputeSplitScore(bst_uint nodeid,
   return ComputeSplitScore(nodeid, featureid, left_stats, right_stats, left_weight, right_weight);
 }
 
+bool SplitEvaluator::CheckFeatureConstraint(bst_uint nodeid, bst_uint featureid) const {
+  return true;
+}
+
 //! \brief Encapsulates the parameters for ElasticNet
 struct ElasticNetParams : public dmlc::Parameter<ElasticNetParams> {
   bst_float reg_lambda;
@@ -153,6 +157,10 @@ class ElasticNet final : public SplitEvaluator {
     return w;
   }
 
+  bool CheckFeatureConstraint(bst_uint nodeid, bst_uint featureid) const override {
+    return true;
+  }
+
  private:
   ElasticNetParams params_;
 
@@ -297,6 +305,10 @@ class MonotonicConstraint final : public SplitEvaluator {
     }
   }
 
+  bool CheckFeatureConstraint(bst_uint nodeid, bst_uint featureid) const override {
+    return true;
+  }
+
  private:
   MonotonicConstraintParams params_;
   std::unique_ptr<SplitEvaluator> inner_;
@@ -481,6 +493,10 @@ class InteractionConstraint final : public SplitEvaluator {
     }
   }
 
+  bool CheckFeatureConstraint(bst_uint nodeid, bst_uint featureid) const override {
+    return CheckInteractionConstraint(featureid, nodeid);
+  }
+
  private:
   InteractionConstraintParams params_;
   std::unique_ptr<SplitEvaluator> inner_;
@@ -500,7 +516,7 @@ class InteractionConstraint final : public SplitEvaluator {
   inline bool CheckInteractionConstraint(bst_uint featureid, bst_uint nodeid) const {
     // short-circuit if no constraint is specified
     return (params_.interaction_constraints.empty()
-            || int_cont_[nodeid].count(featureid) > 0);
+            || int_cont_.at(nodeid).count(featureid) > 0);
   }
 };
 
