@@ -37,6 +37,6 @@ DMLC_REGISTRY_LINK_TAG(updater_sync);
 #ifdef XGBOOST_USE_CUDA
 DMLC_REGISTRY_LINK_TAG(updater_gpu);
 DMLC_REGISTRY_LINK_TAG(updater_gpu_hist);
-#endif
+#endif  // XGBOOST_USE_CUDA
 }  // namespace tree
 }  // namespace xgboost
